@@ -1,7 +1,7 @@
-/* $Id: cpustatus.cc 873 2011-01-02 17:37:23Z dreibh $
+/* $Id: cpustatus.cc 1149 2011-11-26 18:01:43Z michael.tuexen $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2011 by Thomas Dreibholz
+ * Copyright (C) 2009-2012 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,9 @@
 #ifdef __FreeBSD__
 #include <sys/proc.h>
 #endif
+#ifdef __APPLE__
+#include <mach/mach.h>
+#endif
 #include <iostream>
 
 
@@ -45,6 +48,11 @@ const char* CPUStatus::CpuStateNames[] = {
    "User", "Nice", "System", "Idle", "IOWait",
    "Hardware Interrupts", "Software Interrupts", "Hypervisor"
 };
+#elif defined __APPLE__
+#define IDLE_INDEX 2
+const char* CPUStatus::CpuStateNames[] = {
+   "User", "System", "Idle", "Nice"
+};
 #endif
 
 #ifdef __FreeBSD__
@@ -69,36 +77,52 @@ CPUStatus::CPUStatus()
 {
    // ====== Initialize =====================================================
 #ifdef __FreeBSD__
-   int maxCPUs;
    CpuStates = CPUSTATES;
-   getSysCtl("kern.smp.maxcpus", &maxCPUs, sizeof(maxCPUs));
+   getSysCtl("hw.ncpu", &CPUs, sizeof(CPUs));
 #elif defined __linux__
    CpuStates = 8;
    CPUs = sysconf(_SC_NPROCESSORS_CONF);
    if(CPUs < 1) {
       CPUs = 1;
    }
-   const int maxCPUs = CPUs;
 
    ProcStatFD = fopen("/proc/stat", "r");
    if(ProcStatFD == NULL) {
       std::cerr << "ERROR: Unable to open /proc/stat!" << std::endl;
       exit(1);
    }
+#elif defined __APPLE__
+#ifdef USE_PER_CPU_STATISTICS
+   kern_return_t kr;
+   mach_msg_type_number_t count;
+   host_basic_info_data_t hinfo;
+#endif
+
+   CpuStates = CPU_STATE_MAX;
+   if ((host = mach_host_self()) == MACH_PORT_NULL) {
+      std::cerr << "ERROR: Couldn't receive send rights." << std::endl;
+      exit(1);
+   }
+#ifdef USE_PER_CPU_STATISTICS
+   if((kr = host_get_host_priv_port(host, &host_priv)) != KERN_SUCCESS) {
+      mach_error("host_get_host_priv_port():", kr);
+      exit(1);
+   }
+   count = HOST_BASIC_INFO_COUNT;
+   if((kr = host_info(host, HOST_BASIC_INFO, (host_info_t)&hinfo, &count)) != KERN_SUCCESS) {
+      mach_error("host_info():", kr);
+      exit(1);
+   };
+   CPUs = hinfo.max_cpus;
+#else
+   CPUs = 1;
+#endif
 #endif
 
    // ====== Allocate current times array ===================================
-   size_t cpuTimesSize = sizeof(tick_t) * (maxCPUs + 1) * CpuStates;
+   size_t cpuTimesSize = sizeof(tick_t) * (CPUs + 1) * CpuStates;
    CpuTimes = (tick_t*)calloc(1, cpuTimesSize);
    assert(CpuTimes != NULL);
-#ifdef __FreeBSD__
-   if(sysctlbyname("kern.cp_times", CpuTimes, &cpuTimesSize, NULL, 0) < 0) {
-      std::cerr << "ERROR: sysctlbyname(kern.cp_times) failed: "
-                << strerror(errno) << std::endl;
-      exit(1);
-   }
-   CPUs = cpuTimesSize / CpuStates / sizeof(tick_t);
-#endif
 
    // ====== Allocate old times array =======================================
    cpuTimesSize = sizeof(tick_t) * (CPUs + 1) * CpuStates;
@@ -122,16 +146,22 @@ CPUStatus::CPUStatus()
 // ###### Destructor ########################################################
 CPUStatus::~CPUStatus()
 {
-   delete CpuTimes;
+   free(CpuTimes);
    CpuTimes = NULL;
-   delete OldCpuTimes;
+   free(OldCpuTimes);
    OldCpuTimes = NULL;
-   delete Percentages;
+   free(Percentages);
    Percentages = NULL;
 #ifdef __linux__
    fclose(ProcStatFD);
    ProcStatFD = NULL;
 #endif
+#ifdef __APPLE__
+#ifdef USE_PER_CPU_STATISTICS
+   mach_port_deallocate(mach_task_self(), host_priv);
+#endif
+   mach_port_deallocate(mach_task_self(), host);
+#endif
 }
 
 
@@ -144,19 +174,13 @@ void CPUStatus::update()
 
 
    // ====== Get counters ===================================================
-   unsigned int i, j;
 #ifdef __FreeBSD__
    cpuTimesSize = sizeof(tick_t) * CPUs * CpuStates;   /* Total is calculated later! */
-   if(sysctlbyname("kern.cp_times", (tick_t*)&CpuTimes[CpuStates], &cpuTimesSize, NULL, 0) < 0) {
-      std::cerr << "ERROR: sysctlbyname(kern.cp_times) failed: " << strerror(errno) << std::endl;
-      exit(1);
-   }
+   getSysCtl("kern.cp_times", &CpuTimes[CpuStates], cpuTimesSize);
    // ------ Compute total values -------------------------
-   for(j = 0; j < CpuStates; j++) {
+   for(unsigned int j = 0; j < CpuStates; j++) {
       CpuTimes[j] = 0;
-   }
-   for(i = 0; i < CPUs; i++) {
-      for(j = 0; j < CpuStates; j++) {
+      for(unsigned int i = 0; i < CPUs; i++) {
          CpuTimes[j] += CpuTimes[((i + 1) * CpuStates) + j];
       }
    }
@@ -201,24 +225,72 @@ void CPUStatus::update()
          exit(1);
       }
    }
+
+#elif __APPLE__
+   kern_return_t kr;
+#ifdef USE_PER_CPU_STATISTICS
+   processor_port_array_t processor_list;
+   natural_t processor_count, info_count;
+   processor_cpu_load_info_data_t cpu_load_info;
+
+   if((kr = host_processors(host_priv, &processor_list, &processor_count)) != KERN_SUCCESS) {
+      mach_error("host_processors():", kr);
+      exit(1);
+   }
+   for(unsigned int i = 0; i < processor_count; i++) {
+      info_count = PROCESSOR_CPU_LOAD_INFO_COUNT;
+      if ((kr = processor_info(processor_list[i], PROCESSOR_CPU_LOAD_INFO, &host, (processor_info_t)&cpu_load_info, &info_count)) != KERN_SUCCESS) {
+         mach_error("processor_info():", kr);
+         exit(1);
+      }
+      for(unsigned int j = 0; j < CpuStates; j++) {
+         CpuTimes[((i + 1) * CpuStates) + j] = cpu_load_info.cpu_ticks[j];
+      }
+   }
+   vm_deallocate(mach_task_self(), (vm_address_t)processor_list, processor_count * sizeof(processor_t *));
+   // ------ Compute total values -------------------------
+   for(unsigned int j = 0; j < CpuStates; j++) {
+      CpuTimes[j] = 0;
+      for(unsigned int i = 0; i < CPUs; i++) {
+         CpuTimes[j] += CpuTimes[((i + 1) * CpuStates) + j];
+      }
+   }
+#else
+   mach_msg_type_number_t count;
+   host_cpu_load_info_data_t cpu_load_info;
+
+   count = HOST_CPU_LOAD_INFO_COUNT;
+   if((kr = host_statistics(host, HOST_CPU_LOAD_INFO, (host_info_t)&cpu_load_info, &count)) != KERN_SUCCESS) {
+      mach_error("host_statistics():", kr);
+      exit(1);
+   }
+   for(unsigned int j = 0; j < CpuStates; j++) {
+      CpuTimes[j] = CpuTimes[CpuStates + j] = cpu_load_info.cpu_ticks[j];
+   }
+#endif
 #endif
 
 
    // ====== Calculate percentages ==========================================
-   for(i = 0; i < CPUs + 1; i++) {
+   for(unsigned int i = 0; i < CPUs + 1; i++) {
       tick_t diffTotal = 0;
       tick_t diff[CpuStates];
-      for(j = 0; j < CpuStates; j++) {
+      for(unsigned int j = 0; j < CpuStates; j++) {
          const unsigned int index = (i * CpuStates) + j;
-         diff[j] = CpuTimes[index] - OldCpuTimes[index];
-         if(diff[j] < 0) {   // Counter wrap!
+         if(CpuTimes[index] >= OldCpuTimes[index]) {
+            diff[j] = CpuTimes[index] - OldCpuTimes[index];
+         }
+         else {   // Counter wrap!
             diff[j] = OldCpuTimes[index] - CpuTimes[index];
          }
          diffTotal += diff[j];
       }
-      for(j = 0; j < CpuStates; j++) {
+      for(unsigned int j = 0; j < CpuStates; j++) {
          const unsigned int index = (i * CpuStates) + j;
-         Percentages[index] = 100.0 * (float)diff[j] / (float)diffTotal;
+         if(diffTotal != 0) {   // Avoid division by zero!
+            Percentages[index] = 100.0 * (float)diff[j] / (float)diffTotal;
+         }
+         assert( (Percentages[index] >= 0.0) && (Percentages[index] <= 100.0) );
       }
    }
 }
