@@ -1,7 +1,7 @@
-/* $Id: flow.cc 939 2011-01-28 12:03:50Z dreibh $
+/* $Id: flow.cc 1140 2011-11-23 06:43:27Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2011 by Thomas Dreibholz
+ * Copyright (C) 2009-2012 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -397,13 +397,11 @@ void FlowManager::handleEvents(const unsigned long long now)
    lock();
 
    // ====== Handle statistics events =======================================
-   bool hasUpdate = false;
    for(std::map<uint64_t, Measurement*>::iterator iterator = MeasurementSet.begin();
        iterator != MeasurementSet.end(); iterator++) {
        Measurement* measurement = iterator->second;
        if(measurement->NextStatisticsEvent <= now) {
           measurement->writeVectorStatistics(now, GlobalStats, RelGlobalStats);
-          hasUpdate = true;
        }
    }
 
@@ -1229,14 +1227,13 @@ bool Flow::configureSocket(const int socketDescriptor)
         }
 #else
         if(TrafficSpec.CMT != NPAF_PRIMARY_PATH) {
-          std::cerr << "ERROR: CMT usage configured, but not supported by this system!" << std::endl;
+          std::cerr << "ERROR: CMT usage on SCTP socket configured, but not supported by this system!" << std::endl;
           return(false);
         }
 #endif
    }
-
 #ifdef HAVE_DCCP
-   if(TrafficSpec.Protocol == IPPROTO_DCCP) {
+   else if(TrafficSpec.Protocol == IPPROTO_DCCP) {
       const uint8_t value = TrafficSpec.CCID;
       if(value != 0) {
          if(ext_setsockopt(socketDescriptor, SOL_DCCP, DCCP_SOCKOPT_CCID, &value, sizeof(value)) < 0) {
@@ -1254,5 +1251,6 @@ bool Flow::configureSocket(const int socketDescriptor)
       }
    }
 #endif
+
    return(true);
 }
