@@ -1,7 +1,7 @@
-/* $Id: flow.h 937 2011-01-28 11:43:11Z dreibh $
+/* $Id: flow.h 1140 2011-11-23 06:43:27Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2011 by Thomas Dreibholz
+ * Copyright (C) 2009-2012 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,15 +34,6 @@
 
 #include <poll.h>
 
-#ifdef HAVE_DCCP
-#include <linux/dccp.h>
-#ifndef SOL_DCCP
-#define SOL_DCCP 269
-#endif
-#else
-#warning DCCP is not supported by the API of this system!
-#endif
-
 #include <vector>
 #include <map>
 
