@@ -1,7 +1,7 @@
-/* $Id: netperfmeter.cc 939 2011-01-28 12:03:50Z dreibh $
+/* $Id: netperfmeter.cc 1140 2011-11-23 06:43:27Z dreibh $
  *
  * Network Performance Meter
- * Copyright (C) 2009-2011 by Thomas Dreibholz
+ * Copyright (C) 2009-2012 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,13 @@
 #include "control.h"
 #include "transfer.h"
 
+#ifdef HAVE_MPTCP
+#warning linux/tcp.h does not work with C++ code. Using ugly workaround!
+// #include <linux/tcp.h>
+#define TCP_MULTIPATH_ENABLE   19   /* MPTCP_ENABLE */
+// FIXME: This should really be set within a standard include file!
+#endif
+
 
 using namespace std;
 
@@ -47,6 +54,7 @@ static const char*   gActiveNodeName  = "Client";
 static const char*   gPassiveNodeName = "Server";
 static int           gControlSocket   = -1;
 static int           gTCPSocket       = -1;
+static int           gMPTCPSocket     = -1;
 static int           gUDPSocket       = -1;
 static int           gSCTPSocket      = -1;
 static int           gDCCPSocket      = -1;
@@ -247,12 +255,33 @@ static const char* parseTrafficSpecOption(const char*      parameters,
          trafficSpec.CMT = NPAF_PRIMARY_PATH;
          n = 4 + 3;
       }
-      else if(strncmp((const char*)&parameters[4], "normal", 6) == 0) {
+      else if(strncmp((const char*)&parameters[4], "cmtrpv1", 7) == 0) {
+         trafficSpec.CMT = NPAF_CMTRPv1;
+         n = 4 + 7;
+      }
+      else if(strncmp((const char*)&parameters[4], "cmtrpv2", 7) == 0) {
+         trafficSpec.CMT = NPAF_CMTRPv2;
+         n = 4 + 7;
+      }
+      else if(strncmp((const char*)&parameters[4], "cmt", 3) == 0) {
+         trafficSpec.CMT = NPAF_CMT;
+         n = 4 + 3;
+      }
+      else if( (strncmp((const char*)&parameters[4], "like-mptcp", 10) == 0) ||
+               (strncmp((const char*)&parameters[4], "mptcp-like", 10) == 0) ) {
+         trafficSpec.CMT = NPAF_LikeMPTCP;
+         n = 4 + 10;
+      }
+      else if(strncmp((const char*)&parameters[4], "mptcp", 5) == 0) {
+         trafficSpec.CMT = NPAF_LikeMPTCP;
+         n = 4 + 5;
+      }
+      else if(strncmp((const char*)&parameters[4], "normal", 6) == 0) {   // Legacy: use "cmt"!
          trafficSpec.CMT = NPAF_CMT;
          n = 4 + 6;
       }
-      else if(strncmp((const char*)&parameters[4], "rp", 2) == 0) {
-         trafficSpec.CMT = NPAF_CMTRP;
+      else if(strncmp((const char*)&parameters[4], "rp", 2) == 0) {   // Legacy: use "cmtrpv1"!
+         trafficSpec.CMT = NPAF_CMTRPv1;
          n = 4 + 2;
       }
       else if(sscanf((const char*)&parameters[4], "%u%n", &cmt, &pos) == 1) {
@@ -263,6 +292,22 @@ static const char* parseTrafficSpecOption(const char*      parameters,
          cerr << "ERROR: Invalid \"cmt\" setting: " << (const char*)&parameters[4] << "!" << std::endl;
          exit(1);
       }
+
+      // ------ Set correct "pseudo-protocol" for MPTCP ---------------------
+      if( (trafficSpec.Protocol == IPPROTO_TCP) ||
+          (trafficSpec.Protocol == IPPROTO_MPTCP) ) {
+         if(trafficSpec.CMT == NPAF_PRIMARY_PATH) {
+            trafficSpec.Protocol = IPPROTO_TCP;
+         }
+         else {
+            trafficSpec.Protocol = IPPROTO_MPTCP;
+            if(trafficSpec.CMT != NPAF_LikeMPTCP) {
+               cerr << "WARNING: Invalid \"cmt\" setting: " << (const char*)&parameters[4]
+                    << " for MPTCP! Using default instead!" << std::endl;
+            }
+         }
+      }
+      // --------------------------------------------------------------------
    }
    else if(strncmp(parameters, "ccid=", 5) == 0) {
       unsigned int ccid;
@@ -337,8 +382,8 @@ static Flow* createFlow(Flow*                  previousFlow,
                         const uint64_t         measurementID,
                         const char*            vectorNamePattern,
                         const OutputFileFormat vectorFileFormat,
-                        const uint8_t          protocol,
-                        const sockaddr*        remoteAddress)
+                        const uint8_t          initialProtocol,
+                        const sockaddr_union&  remoteAddress)
 {
    // ====== Get flow ID and stream ID ======================================
    static uint32_t flowID   = 0; // will be increased with each successfull call
@@ -346,12 +391,13 @@ static Flow* createFlow(Flow*                  previousFlow,
 
    // ====== Get FlowTrafficSpec ============================================
    FlowTrafficSpec trafficSpec;
-   trafficSpec.Protocol = protocol;
+   trafficSpec.Protocol = initialProtocol;
    if(strncmp(parameters, "default", 7) == 0) {
       trafficSpec.OutboundFrameRateRng = RANDOM_CONSTANT;
       trafficSpec.OutboundFrameRate[0] = 0.0;
       switch(trafficSpec.Protocol) {
          case IPPROTO_TCP:
+         case IPPROTO_MPTCP:
             trafficSpec.OutboundFrameSize[0] = 1500 - 40 - 20;
           break;
          case IPPROTO_UDP:
@@ -425,42 +471,57 @@ static Flow* createFlow(Flow*                  previousFlow,
    else {
       originalSocketDescriptor = true;
       socketDescriptor          = -1;
-      switch(protocol) {
+      switch(trafficSpec.Protocol) {
          case IPPROTO_SCTP:
-            socketDescriptor = createAndBindSocket(remoteAddress->sa_family, SOCK_STREAM, IPPROTO_SCTP, 0,
+            socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_SCTP, 0,
                                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
            break;
          case IPPROTO_TCP:
-            socketDescriptor = createAndBindSocket(remoteAddress->sa_family, SOCK_STREAM, IPPROTO_TCP, 0,
+            socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_TCP, 0,
                                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
            break;
+#ifdef HAVE_MPTCP
+         case IPPROTO_MPTCP:
+            socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_MPTCP, 0,
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+           break;
+#endif
          case IPPROTO_UDP:
-            socketDescriptor = createAndBindSocket(remoteAddress->sa_family, SOCK_DGRAM, IPPROTO_UDP, 0,
+            socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_DGRAM, IPPROTO_UDP, 0,
                                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
            break;
 #ifdef HAVE_DCCP
          case IPPROTO_DCCP:
-            socketDescriptor = createAndBindSocket(remoteAddress->sa_family, SOCK_DCCP, IPPROTO_DCCP, 0,
+            socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_DCCP, IPPROTO_DCCP, 0,
                                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
            break;
 #endif
+         default:
+            abort();
+          break;
       }
       if(socketDescriptor < 0) {
-         cerr << "ERROR: Unable to create " << getProtocolName(protocol)
+         cerr << "ERROR: Unable to create " << getProtocolName(trafficSpec.Protocol)
               << " socket - " << strerror(errno) << "!" << endl;
          exit(1);
       }
 
+      // ====== MPTCP: choose MPTCP socket instead of TCP socket ============
+      sockaddr_union destinationAddress = remoteAddress;
+      if(trafficSpec.Protocol == IPPROTO_MPTCP) {
+         setPort(&destinationAddress.sa, getPort(&destinationAddress.sa) - 1);
+      }
+
       // ====== Establish connection ========================================
       if(gOutputVerbosity >= NPFOV_STATUS) {
          cout << "Flow #" << flow->getFlowID() << ": connecting "
-            << getProtocolName(protocol) << " socket to ";
-         printAddress(cout, remoteAddress, true);
+            << getProtocolName(trafficSpec.Protocol) << " socket to ";
+         printAddress(cout, &destinationAddress.sa, true);
          cout << " ... ";
          cout.flush();
       }
 
-      if(protocol == IPPROTO_SCTP) {
+      if(trafficSpec.Protocol == IPPROTO_SCTP) {
          sctp_initmsg initmsg;
          memset((char*)&initmsg, 0 ,sizeof(initmsg));
          initmsg.sinit_num_ostreams  = 65535;
@@ -486,9 +547,8 @@ static Flow* createFlow(Flow*                  previousFlow,
       if(flow->configureSocket(socketDescriptor) == false) {
          exit(1);
       }
-
-      if(ext_connect(socketDescriptor, remoteAddress, getSocklen(remoteAddress)) < 0) {
-         cerr << "ERROR: Unable to connect " << getProtocolName(protocol)
+      if(ext_connect(socketDescriptor, &destinationAddress.sa, getSocklen(&destinationAddress.sa)) < 0) {
+         cerr << "ERROR: Unable to connect " << getProtocolName(trafficSpec.Protocol)
               << " socket - " << strerror(errno) << "!" << endl;
          exit(1);
       }
@@ -531,6 +591,7 @@ bool mainLoop(const bool               isActiveMode,
    int                    n         = 0;
    int                    controlID = -1;
    int                    tcpID     = -1;
+   int                    mptcpID   = -1;
    int                    udpID     = -1;
    int                    sctpID    = -1;
    int                    dccpID    = -1;
@@ -540,10 +601,11 @@ bool mainLoop(const bool               isActiveMode,
 
    // ====== Get parameters for poll() ======================================
    addToPollFDs((pollfd*)&fds, gControlSocket, n, &controlID);
-   addToPollFDs((pollfd*)&fds, gTCPSocket,  n, &tcpID);
-   addToPollFDs((pollfd*)&fds, gUDPSocket,  n, &udpID);
-   addToPollFDs((pollfd*)&fds, gSCTPSocket, n, &sctpID);
-   addToPollFDs((pollfd*)&fds, gDCCPSocket, n, &dccpID);
+   addToPollFDs((pollfd*)&fds, gTCPSocket,     n, &tcpID);
+   addToPollFDs((pollfd*)&fds, gMPTCPSocket,   n, &mptcpID);
+   addToPollFDs((pollfd*)&fds, gUDPSocket,     n, &udpID);
+   addToPollFDs((pollfd*)&fds, gSCTPSocket,    n, &sctpID);
+   addToPollFDs((pollfd*)&fds, gDCCPSocket,    n, &dccpID);
 
 
    // ====== Use poll() to wait for events ==================================
@@ -576,6 +638,12 @@ bool mainLoop(const bool               isActiveMode,
             FlowManager::getFlowManager()->addSocket(IPPROTO_TCP, newSD);
          }
       }
+      if( (mptcpID >= 0) && (fds[mptcpID].revents & POLLIN) ) {
+         const int newSD = ext_accept(gMPTCPSocket, NULL, 0);
+         if(newSD >= 0) {
+            FlowManager::getFlowManager()->addSocket(IPPROTO_MPTCP, newSD);
+         }
+      }
       if( (udpID >= 0) && (fds[udpID].revents & POLLIN) ) {
          FlowManager::getFlowManager()->lock();
          handleNetPerfMeterData(isActiveMode, now, IPPROTO_UDP, gUDPSocket);
@@ -648,6 +716,31 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
       exit(1);
    }
 
+#ifdef HAVE_MPTCP
+   gMPTCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_MPTCP, localPort - 1,
+                                      gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+   if(gMPTCPSocket < 0) {
+      cerr << "ERROR: Failed to create and bind MPTCP socket - "
+           << strerror(errno) << "!" << endl;
+      exit(1);
+   }
+   int cmtOnOff = 1;
+   if(ext_setsockopt(gMPTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
+      std::cerr << "NOTE: Compiled with MPTCP support, but unable to initialize it: "
+                << strerror(errno) << "!" << endl;
+      ext_close(gMPTCPSocket);
+      gMPTCPSocket = -1;
+   }
+   else {
+      cmtOnOff = 0;
+      if(ext_setsockopt(gTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
+         cerr << "ERROR: Failed to disable MPTCP on TCP socket - "
+              << strerror(errno) << "!" << endl;
+         exit(1);
+      }
+   }
+#endif
+
    gUDPSocket = createAndBindSocket(AF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP, localPort,
                                     gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
    if(gUDPSocket < 0) {
@@ -699,7 +792,9 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
 
    // ====== Print status ===================================================
-   cout << "Passive Mode: Accepting TCP/UDP/SCTP" << ((gDCCPSocket > 0) ? "/DCCP" : "")
+   cout << "Passive Mode: Accepting TCP"
+        << ((gMPTCPSocket > 0) ? "+MPTCP" : "")
+        << "/UDP/SCTP" << ((gDCCPSocket > 0) ? "/DCCP" : "")
         << " connections on port " << localPort << endl << endl;
 
 
@@ -721,6 +816,9 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
    gMessageReader.deregisterSocket(gControlSocket);
    ext_close(gControlSocket);
    ext_close(gTCPSocket);
+   if(gMPTCPSocket >= 0) {
+      ext_close(gMPTCPSocket);
+   }
    FlowManager::getFlowManager()->removeSocket(gUDPSocket, false);
    ext_close(gUDPSocket);
    ext_close(gSCTPSocket);
@@ -740,9 +838,12 @@ void activeMode(int argc, char** argv)
       cerr << "ERROR: Invalid remote address " << argv[1] << "!" << endl;
       exit(1);
    }
-   if(getPort(&remoteAddress.sa) == 0) {
+   if(getPort(&remoteAddress.sa) < 2) {
       setPort(&remoteAddress.sa, 9000);
    }
+   else if(getPort(&remoteAddress.sa) > 65534) {
+      setPort(&remoteAddress.sa, 65534);
+   }
    sockaddr_union controlAddress = remoteAddress;
    setPort(&controlAddress.sa, getPort(&remoteAddress.sa) + 1);
 
@@ -856,7 +957,7 @@ void activeMode(int argc, char** argv)
 
          lastFlow = createFlow(lastFlow, argv[i], measurementID,
                                vectorNamePattern, vectorFileFormat,
-                               protocol, &remoteAddress.sa);
+                               protocol, remoteAddress);
          hasFlow = true;
 
          if(!performNetPerfMeterAddFlow(&gMessageReader, gControlSocket, lastFlow)) {
@@ -894,7 +995,6 @@ void activeMode(int argc, char** argv)
    // ====== Main loop ======================================================
    const unsigned long long stopAt  = (gRuntime > 0) ?
       (getMicroTime() + (unsigned long long)rint(gRuntime * 1000000.0)) : ~0ULL;
-   bool                     aborted = false;
    signal(SIGPIPE, SIG_IGN);
    installBreakDetector();
    if(gOutputVerbosity >= NPFOV_BANDWIDTH_INFO) {
@@ -904,7 +1004,6 @@ void activeMode(int argc, char** argv)
    while( (!breakDetected()) && (!gStopTimeReached) ) {
       if(!mainLoop(true, stopAt, measurementID)) {
          cout << endl << "*** Aborted ***" << endl;
-         aborted = true;
          break;
       }
    }
